//
//	SPK-FM-301(AR1000) FM Reciever
//
//	MCU:		ATtiny2313
//
//		efuse:0xFF	自己プログラミング不許可
//		hfuse:0x9D	BOD 1.8V チップ消去から EEPROM保護
//		lfuse:0xE2	内蔵CR 4MHz
//
//	Compiler:	AVR-GCC		4.2.2
//				AVR-LIBC	1.4.7
//				binutils	2.18
//
//	2008.10.4	KAWAKAMI Yukio
//

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

#include "AR1000.h"

#define	CPU_CLOCK	4000000L

#define	TEST_FREQ	807		// 80.7MHz FMながおか

volatile uchar	Timer;

PROGMEM uchar VolumeValue[] = {
	0x0F, 0x0E, 0x0D, 0x0B,
	0x0A, 0x09, 0x07, 0x06,
	0x05, 0x03, 0x02, 0x01,
	0x00
};

//*******************************************************
//	タイマー１比較一致Ａ割り込み
//	100Hzで割り込みがかかる
//*******************************************************
ISR(TIMER1_COMPA_vect){

	if (Timer)	--Timer;
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main(void){
	
	cli();

	PORTB = 0xFF;
	DDRB = 0;
	
	PORTD = (1<<PD6)|(1<<PD5)|(1<<PD4)|(1<<SCL)|(1<<SDA);
	DDRD = (1<<LED);

	MCUCR = (1<<SE);	// スリープ許可・アイドルモード
	
	// タイマ１
	TCCR1A = 0;						// CTC動作
	TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10);	// プリスケーラ 1/1024
	OCR1AH = (CPU_CLOCK/1024/100 - 1) >> 8;
	OCR1AL = (uchar)(CPU_CLOCK/1024/100 - 1);	// 100Hz
	TCNT1H = 0;
	TCNT1L = 0;
	TIMSK = (1<<OCIE1A);			// 比較A割り込み許可

	ACSR = (1<<ACD);			// アナログコンパレータ禁止
	
	sei();

	// １秒時間待ち(電源が安定するまで待つ)
	wait_1s();


	// AR1000 レジスタ初期値
	write_AR1000(R0, R0_DEFAULT | R0_XO_EN);	// 最初は必ず ENABLE=OFF
	write_AR1000(R1, R1_DEFAULT | R1_HMUTE);	// ミュート
	write_AR1000(R2, R2_DEFAULT);
	write_AR1000(R3, R3_DEFAULT);
	write_AR1000(R4, 0x0400);
	write_AR1000(R5, 0x28AA);
	write_AR1000(R6, 0x4400);
	write_AR1000(R7, 0x1EE7);
	write_AR1000(R8, 0x7141);
	write_AR1000(R9, 0x007D);
	write_AR1000(R10, 0x82CE);
	write_AR1000(R11, 0x4F55);
	write_AR1000(R12, 0x970C);
	write_AR1000(R13, 0xB845);
	write_AR1000(R14, 0xFC2D);
	write_AR1000(R15, 0x8097);
	write_AR1000(R16, 0x04A1);
	write_AR1000(R17, 0xDF6A);

	write_AR1000(R0, R0_DEFAULT | R0_XO_EN | R0_ENABLE);	// ここで ENABLE

	for (;;){
		wait_10ms(1);	// 10ms WAIT
		if (read_AR1000(STATUS) & 0x20)	break;	// STC=H 待ち
	}

	write_AR1000(R2, (R2_DEFAULT & ~(R2_TUNE | R2_CHAN))	// 周波数設定
				 | (TEST_FREQ - 690));
	write_AR1000(R2, (R2_DEFAULT & ~(R2_TUNE | R2_CHAN))
				 | (TEST_FREQ - 690) | R2_TUNE);			// TUNE=ON
	
	for (;;){
		wait_10ms(1);	// 10ms WAIT
		if (read_AR1000(STATUS) & STATUS_STC)	break;	// STC=H 待ち
	}

	write_AR1000(R1, R1_DEFAULT);	// ミュート解除

	PORTD |= (1<<LED);

	for (;;){
		wait_1s();
		PORTD ^= (1<<LED);
	}
}


//*******************************************************
//	AR1000 レジスタ書き込み
//*******************************************************
void write_AR1000(uchar adrs, ushort data){

	TWI_start();
	TWI_send(0x20);			// スレーブアドレス + Write
	TWI_send(adrs);
	TWI_send(data >> 8);
	TWI_send(data & 0x00FF);
	TWI_stop();
}


//*******************************************************
//	AR1000 レジスタ読み込み
//*******************************************************
ushort read_AR1000(uchar adrs){
	ushort	data;

	TWI_start();
	TWI_send(0x20);			// スレーブアドレス + Write
	TWI_send(adrs);
	TWI_start();
	TWI_send(0x20 + 1);		// スレーブアドレス + Read
	data = TWI_recv(0);		// ACK
	data <<= 8;
	data |= TWI_recv(1);	// NACK
	TWI_stop();

	return data;
}


//*******************************************************
//	1秒時間待ち
//*******************************************************
void wait_1s(void){

	Timer = 100;
	while(Timer){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	10ms単位で時間待ち
//*******************************************************
void wait_10ms(uchar w){

	Timer = w;
	while(Timer){
		WDR;
		SLEEP;
	}
}


//************************************************************
//	TWI 開始条件
//************************************************************
void TWI_start(void){

	SDA_H;
	NOP;
	SCL_H;
	NOP;
	SDA_L;
}


//************************************************************
//	TWI 8bit 送信
//************************************************************
void TWI_send(uchar data){
	char	bit;

	SCL_L;
	for (bit = 0; bit < 8; bit++){
		if (data & 0x80){
			SDA_H;
		} else {
			SDA_L;
		}
		SCL_H;
		WAIT_SCL_H;		// SCL=Hになるまで待つ
		data <<= 1;
		SCL_L;
	}

	SDA_H;
	SCL_L;

	// ACK受信 内容は無視
	SCL_H;
	NOP;
	SCL_L;
}


//************************************************************
//	TWI 8bit 受信
//************************************************************
uchar TWI_recv(uchar nack){
	char	bit;
	uchar	data;

	SDA_H;
	SCL_L;

	data = 0;
	for (bit = 0; bit < 8; bit++){
		SCL_H;
		WAIT_SCL_H;		// SCL=Hになるまで待つ
		data <<= 1;
		if (INP_SDA){
			data |= 1;
		}
		SCL_L;
	}

	// ACK/NACK 送信
	if (nack){
		SCL_L;
		SCL_H;
		SDA_H;
	} else {
		SDA_L;
		SCL_L;
		SCL_H;
	}

	SCL_L;
	SDA_H;

	return data;
}


//************************************************************
//	TWI 停止条件
//************************************************************
void TWI_stop(void){

	SDA_L;
	NOP;
	SCL_H;
	NOP;
	SDA_H;
}
